<?php

namespace Drupal\commerce_bulk\Plugin\Action;

use Drupal\Core\Action\ConfigurableActionBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Activate or deactivate variation.
 *
 * @Action(
 *   id = "commerce_bulk_variation_status",
 *   label = @Translation("Change status"),
 *   type = "commerce_product_variation"
 * )
 */
class VariationStatus extends ConfigurableActionBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $request = \Drupal::request();
    $storage = \Drupal::service('entity_type.manager')->getStorage('commerce_product_variation');
    if ($ids = explode('|', $request->query->get('ids'))) {
      $count = count($ids);
      $form_state->set('variations', $storage->loadMultiple($ids));
      $form['warning'] = [
        '#markup' => new TranslatableMarkup('<h1>Change status of the <span style="color:red;font-weight:bolder;">@count</span> @variations.</h1>', [
          '@count' => $count,
          '@variations' => $count > 1 ? $this->t('variations') : $this->t('variation'),
        ]),
      ];
      $form['status'] = [
        '#type' => 'radios',
        '#options' => [
          1 => $this->t('Activate'),
          0  => $this->t('Deactivate'),
        ],
        '#default_value' => 1,
      ];
      $form['cancel'] = [
      '#type' => 'submit',
      '#value' => 'CANCEL AND BACK',
      '#weight' => 1000,
      ];
      // Remove the "Action was applied to N items" message.
      \Drupal::messenger()->deleteByType('status');
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getTriggeringElement()['#id'] != 'edit-cancel') {
      $status = (bool) $form_state->getValue('status');
      foreach ($form_state->get('variations') as $variation) {
        $variation->setActive($status)->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function executeMultiple(array $variations) {
    if ($variations) {
      $ids = [];
      foreach ($variations as $variation) {
        $ids[] = $variation->id();
      }
      $url = $variation->toUrl();
      $query = ['destination' => \Drupal::request()->getRequestUri(), 'ids' => implode('|', $ids),];
      $path = $url::fromUserInput('/admin/config/system/actions/configure/' . $this->getPluginId(), ['query' => $query])->toString();
      $response = new RedirectResponse($path);
      $response->send();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function execute($variation = NULL) {
    // Do nothing.
  }

  /**
   * {@inheritdoc}
   */
  public function access($variation, AccountInterface $account = NULL, $return_as_object = FALSE) {
    $result = $variation->access('update', $account, TRUE);

    return $return_as_object ? $result : $result->isAllowed();
  }

}
